@@ -1,7 +1,7 @@
 """Property is a single CSS property in a CSSStyleDeclaration."""
 __all__ = ['Property']
 __docformat__ = 'restructuredtext'
-__version__ = '$Id: property.py 1811 2009-07-29 13:11:15Z cthedot $'
+__version__ = '$Id: property.py 1978 2010-05-29 23:40:59Z cthedot $'
 
 from cssutils.helper import Deprecated
 from cssvalue import CSSValue
@@ -22,8 +22,8 @@ class Property(cssutils.util.Base):
           ;
         term
           : unary_operator?
-            [ NUMBER S* | PERCENTAGE S* | LENGTH S* | EMS S* | EXS S* | ANGLE S* |
-              TIME S* | FREQ S* | function ]
+            [ NUMBER S* | PERCENTAGE S* | LENGTH S* | EMS S* | EXS S* | 
+              ANGLE S* | TIME S* | FREQ S* | function ]
           | STRING S* | IDENT S* | URI S* | hexcolor
           ;
         function
@@ -43,8 +43,8 @@ class Property(cssutils.util.Base):
           ;
 
     """
-    def __init__(self, name=None, value=None, priority=u'', 
-                 _mediaQuery=False, parent=None, parentStyle=None):
+    def __init__(self, name=None, value=None, priority=u'',
+                 _mediaQuery=False, parent=None):
         """
         :param name:
             a property name string (will be normalized)
@@ -56,10 +56,8 @@ class Property(cssutils.util.Base):
         :param _mediaQuery:
             if ``True`` value is optional (used by MediaQuery)
         :param parent:
-            the parent object, normally a 
+            the parent object, normally a
             :class:`cssutils.css.CSSStyleDeclaration`
-        :param parentStyle:
-            DEPRECATED: Use ``parent`` instead
         """
         super(Property, self).__init__()
         self.seqs = [[], None, []]
@@ -70,29 +68,32 @@ class Property(cssutils.util.Base):
         self.__nametoken = None
         self._name = u''
         self._literalname = u''
+        self.seqs[1] = CSSValue(parent=self)
         if name:
             self.name = name
-
-        if value:
             self.cssValue = value
-        else:
-            self.seqs[1] = CSSValue()
 
         self._priority = u''
         self._literalpriority = u''
         if priority:
             self.priority = priority
-            
+
     def __repr__(self):
-        return "cssutils.css.%s(name=%r, value=%r, priority=%r)" % (
+        return u"cssutils.css.%s(name=%r, value=%r, priority=%r)" % (
                 self.__class__.__name__,
-                self.literalname, self.cssValue.cssText, self.priority)
+                self.literalname,
+                self.cssValue.cssText,
+                self.priority)
 
     def __str__(self):
-        return "<%s.%s object name=%r value=%r priority=%r valid=%r at 0x%x>" % (
-                self.__class__.__module__, self.__class__.__name__,
-                self.name, self.cssValue.cssText, self.priority,
-                self.valid, id(self))
+        return u"<%s.%s object name=%r value=%r priority=%r valid=%r at 0x%x>" \
+               % (self.__class__.__module__,
+                  self.__class__.__name__,
+                  self.name,
+                  self.cssValue.cssText,
+                  self.priority,
+                  self.valid,
+                  id(self))
 
     def _getCssText(self):
         """Return serialized property cssText."""
@@ -145,12 +146,12 @@ class Property(cssutils.util.Base):
                 self._log.error(u'Property: No property value found: %r.' %
                                 self._valuestr(cssText), colontoken)
 
-            if wellformed:                
+            if wellformed:
                 self.wellformed = True
                 self.name = nametokens
                 self.cssValue = valuetokens
                 self.priority = prioritytokens
-                
+
                 # also invalid values are set!
                 self.validate()
 
@@ -163,7 +164,7 @@ class Property(cssutils.util.Base):
 
     def _setName(self, name):
         """
-        :exceptions:    
+        :exceptions:
             - :exc:`~xml.dom.SyntaxErr`:
               Raised if the specified name has a syntax error and is
               unparsable.
@@ -225,7 +226,7 @@ class Property(cssutils.util.Base):
 
     name = property(lambda self: self._name, _setName,
                     doc="Name of this property.")
-    
+
     literalname = property(lambda self: self._literalname,
                            doc="Readonly literal (not normalized) name "
                                "of this property")
@@ -246,31 +247,30 @@ class Property(cssutils.util.Base):
           type of values than the values allowed by the CSS property.
         """
         if self._mediaQuery and not cssText:
-            self.seqs[1] = CSSValue()
+            self.seqs[1] = CSSValue(parent=self)
         else:
-            if not self.seqs[1]:
-                self.seqs[1] = CSSValue()
-
-            cssvalue = self.seqs[1]
-            cssvalue.cssText = cssText
-            if cssvalue.wellformed: #cssvalue._value and 
-                self.seqs[1] = cssvalue
-            self.wellformed = self.wellformed and cssvalue.wellformed
+            try:
+                self.seqs[1].cssText = cssText
+            except xml.dom.SyntaxErr, e:
+                v = CSSValue(parent=self)
+                # try parsing for CSSValue (simply raise if not)
+                # might have been e.g. a CSSVariable before
+                v.cssText = cssText
+                self.seqs[1] = v
+                            
+            self.wellformed = self.wellformed and self.seqs[1].wellformed
 
     cssValue = property(_getCSSValue, _setCSSValue,
         doc="(cssutils) CSSValue object of this property")
 
-
     def _getValue(self):
         if self.cssValue:
-            return self.cssValue.cssText # _value # [0]
+            return self.cssValue.cssText
         else:
             return u''
 
     def _setValue(self, value):
-        self.cssValue.cssText = value
-#        self.valid = self.valid and self.cssValue.valid
-        self.wellformed = self.wellformed and self.cssValue.wellformed
+        self._setCSSValue(value)
 
     value = property(_getValue, _setValue,
                      doc="The textual value of this Properties cssValue.")
@@ -293,14 +293,16 @@ class Property(cssutils.util.Base):
               Raised if the specified priority has a syntax error and is
               unparsable.
               In this case a priority not equal to None, "" or "!{w}important".
-              As CSSOM defines CSSStyleDeclaration.getPropertyPriority resulting in
-              u'important' this value is also allowed to set a Properties priority
+              As CSSOM defines CSSStyleDeclaration.getPropertyPriority resulting
+              in u'important' this value is also allowed to set a Properties
+              priority
         """
         if self._mediaQuery:
             self._priority = u''
             self._literalpriority = u''
             if priority:
-                self._log.error(u'Property: No priority in a MediaQuery - ignored.')
+                self._log.error(u'Property: No priority in a MediaQuery - '
+                                u'ignored.')
             return
 
         if isinstance(priority, basestring) and\
@@ -373,11 +375,11 @@ class Property(cssutils.util.Base):
 
     def validate(self):
         """Validate value against `profiles` which are checked dynamically.
-        properties in e.g. @font-face rules are checked against 
-        ``cssutils.profile.CSS3_FONT_FACE`` only. 
-        
+        properties in e.g. @font-face rules are checked against
+        ``cssutils.profile.CSS3_FONT_FACE`` only.
+
         For each of the following cases a message is reported:
-        
+
         - INVALID (so the property is known but not valid)
             ``ERROR    Property: Invalid value for "{PROFILE-1[/PROFILE-2...]"
             property: ...``
@@ -390,57 +392,65 @@ class Property(cssutils.util.Base):
             ``DEBUG    Found valid "{PROFILE-1[/PROFILE-2...]" property...``
 
         - UNKNOWN property
-            ``WARNING    Unknown Property name...`` is issued            
-            
+            ``WARNING    Unknown Property name...`` is issued
+
         so for example::
-        
+
             cssutils.log.setLevel(logging.DEBUG)
             parser = cssutils.CSSParser()
-            s = parser.parseString('''body { 
+            s = parser.parseString('''body {
                 unknown-property: x;
                 color: 4;
                 color: rgba(1,2,3,4);
-                color: red 
+                color: red
             }''')
 
             # Log output:
-            
+
             WARNING Property: Unknown Property name. [2:9: unknown-property]
             ERROR   Property: Invalid value for "CSS Color Module Level 3/CSS Level 2.1" property: 4 [3:9: color]
             DEBUG   Property: Found valid "CSS Color Module Level 3" value: rgba(1, 2, 3, 4) [4:9: color]
             DEBUG   Property: Found valid "CSS Level 2.1" value: red [5:9: color]
-        
+
 
         and when setting an explicit default profile::
 
             cssutils.profile.defaultProfiles = cssutils.profile.CSS_LEVEL_2
-            s = parser.parseString('''body { 
+            s = parser.parseString('''body {
                 unknown-property: x;
                 color: 4;
                 color: rgba(1,2,3,4);
-                color: red 
+                color: red
             }''')
 
             # Log output:
-            
+
             WARNING Property: Unknown Property name. [2:9: unknown-property]
             ERROR   Property: Invalid value for "CSS Color Module Level 3/CSS Level 2.1" property: 4 [3:9: color]
             WARNING Property: Not valid for profile "CSS Level 2.1" but valid "CSS Color Module Level 3" value: rgba(1, 2, 3, 4)  [4:9: color]
-            DEBUG   Property: Found valid "CSS Level 2.1" value: red [5:9: color]            
+            DEBUG   Property: Found valid "CSS Level 2.1" value: red [5:9: color]
         """
         valid = False
-        
+
         profiles = None
         try:
             # if @font-face use that profile
             rule = self.parent.parentRule
-            if rule.type == rule.FONT_FACE_RULE:
-                profiles = [cssutils.profile.CSS3_FONT_FACE]
-            #TODO: same for @page
         except AttributeError:
             pass
-        
+        else:
+            if rule is not None:
+                if rule.type == rule.FONT_FACE_RULE:
+                    profiles = [cssutils.profile.CSS3_FONT_FACE]
+                #TODO: same for @page
+
         if self.name and self.value:
+            
+            cv = self.cssValue
+            if cv.cssValueType == cv.CSS_VARIABLE and not cv.value:
+                # TODO: false alarms too! 
+                cssutils.log.warn(u'No value for variable "%s" found, keeping '
+                                  u'variable.' % cv.name, neverraise=True)
 
             if self.name in cssutils.profile.knownNames:
                 # add valid, matching, validprofiles...
@@ -448,14 +458,14 @@ class Property(cssutils.util.Base):
                     cssutils.profile.validateWithProfile(self.name,
                                                          self.value,
                                                          profiles)
-                
+
                 if not valid:
                     self._log.error(u'Property: Invalid value for '
                                     u'"%s" property: %s'
                                     % (u'/'.join(validprofiles), self.value),
                                     token=self.__nametoken,
                                     neverraise=True)
-                    
+
                 # TODO: remove logic to profiles!
                 elif valid and not matching:#(profiles and profiles not in validprofiles):
                     if not profiles:
@@ -469,26 +479,17 @@ class Property(cssutils.util.Base):
                                    token = self.__nametoken,
                                    neverraise=True)
                     valid = False
-                                
+
                 elif valid:
                     self._log.debug(u'Property: Found valid "%s" value: %s'
                                    % (u'/'.join(validprofiles), self.value),
                                    token = self.__nametoken,
                                    neverraise=True)
-                    
+
         if self._priority not in (u'', u'important'):
             valid = False
 
         return valid
 
-    valid = property(validate, doc="Check if value of this property is valid "
-                                   "in the properties context.")
-
-
-    @Deprecated('Use ``parent`` attribute instead.')
-    def _getParentStyle(self):
-        return self._parent
-
-    parentStyle = property(_getParentStyle, _setParent,
-        doc="DEPRECATED: Use ``parent`` instead")
-
+    valid = property(validate, doc=u"Check if value of this property is valid "
+                                   u"in the properties context.")
