@@ -3,8 +3,9 @@
 """cssutils serializer"""
 __all__ = ['CSSSerializer', 'Preferences']
 __docformat__ = 'restructuredtext'
-__version__ = '$Id: serialize.py 1741 2009-05-09 18:20:20Z cthedot $'
+__version__ = '$Id: serialize.py 2016 2010-07-06 20:49:41Z cthedot $'
 
+from cssutils.helper import Deprecated, normalize
 import codecs
 import cssutils
 import helper
@@ -15,14 +16,14 @@ def _escapecss(e):
     """
     Escapes characters not allowed in the current encoding the CSS way
     with a backslash followed by a uppercase hex code point
-    
+
     E.g. the german umlaut 'ä' is escaped as \E4
     """
     s = e.object[e.start:e.end]
     return u''.join([ur'\%s ' % str(hex(ord(x)))[2:] # remove 0x from hex
                      .upper() for x in s]), e.end
 
-codecs.register_error('escapecss', _escapecss)               
+codecs.register_error('escapecss', _escapecss)
 
 
 class Preferences(object):
@@ -36,7 +37,7 @@ class Preferences(object):
         the src file, e.g. if ``True``: ``color`` else: ``c\olor``
 
         Only used if ``keepAllProperties==False``.
-        
+
     defaultPropertyPriority = True
         Should the normalized or literal priority be used, e.g. ``!important``
         or ``!Im\portant``
@@ -48,9 +49,9 @@ class Preferences(object):
         Indentation of e.g Properties inside a CSSStyleDeclaration
     indentSpecificities = False
         Indent rules with subset of Selectors and higher Specitivity
-        
+
     keepAllProperties = True
-        If ``True`` all properties set in the original CSSStylesheet 
+        If ``True`` all properties set in the original CSSStylesheet
         are kept meaning even properties set twice with the exact same
         same name are kept!
     keepComments = True
@@ -58,36 +59,43 @@ class Preferences(object):
     keepEmptyRules = False
         defines if empty rules like e.g. ``a {}`` are kept in the resulting
         serialized sheet
-    keepUnkownAtRules = True
-        defines if unknown @rules like e.g. ``@three-dee {}`` are kept in the 
+    keepUnknownAtRules = True
+        defines if unknown @rules like e.g. ``@three-dee {}`` are kept in the
         serialized sheet
     keepUsedNamespaceRulesOnly = False
         if True only namespace rules which are actually used are kept
-        
+
     lineNumbers = False
         Only used if a complete CSSStyleSheet is serialized.
     lineSeparator = u'\\n'
-        How to end a line. This may be set to e.g. u'' for serializing of 
+        How to end a line. This may be set to e.g. u'' for serializing of
         CSSStyleDeclarations usable in HTML style attribute.
     listItemSpacer = u' '
         string which is used in ``css.SelectorList``, ``css.CSSValue`` and
         ``stylesheets.MediaList`` after the comma
+    normalizedVarNames = True
+        defines if variable names should be serialized normalized (they are
+        used as being normalized anyway)
     omitLastSemicolon = True
         If ``True`` omits ; after last property of CSSStyleDeclaration
     paranthesisSpacer = u' '
-        string which is used before an opening paranthesis like in a 
+        string which is used before an opening paranthesis like in a
         ``css.CSSMediaRule`` or ``css.CSSStyleRule``
     propertyNameSpacer = u' '
         string which is used after a Property name colon
+    resolveVariables = True
+        if ``True`` all variable references are tried to resolved and
+        all CSSVariablesRules are removed from the output.
+        Any variable reference not resolvable is simply kept untouched.
     selectorCombinatorSpacer = u' '
         string which is used before and after a Selector combinator like +, > or ~.
         CSSOM defines a single space for this which is also the default in cssutils.
     spacer = u' '
         general spacer, used e.g. by CSSUnknownRule
-    
+
     validOnly = False
         if True only valid (Properties) are output
-        
+
         A Property is valid if it is a known Property with a valid value.
     """
     def __init__(self, **initials):
@@ -98,12 +106,12 @@ class Preferences(object):
                 self.__setattr__(key, value)
 
     def __repr__(self):
-        return u"cssutils.css.%s(%s)" % (self.__class__.__name__, 
+        return u"cssutils.css.%s(%s)" % (self.__class__.__name__,
             u', '.join(['\n    %s=%r' % (p, self.__getattribute__(p)) for p in self.__dict__]
                 ))
 
     def __str__(self):
-        return u"<cssutils.css.%s object %s at 0x%x" % (self.__class__.__name__, 
+        return u"<cssutils.css.%s object %s at 0x%x" % (self.__class__.__name__,
             u' '.join(['%s=%r' % (p, self.__getattribute__(p)) for p in self.__dict__]
                 ),
                 id(self))
@@ -119,21 +127,23 @@ class Preferences(object):
         self.keepAllProperties = True
         self.keepComments = True
         self.keepEmptyRules = False
-        self.keepUnkownAtRules = True
+        self.keepUnknownAtRules = True
         self.keepUsedNamespaceRulesOnly = False
         self.lineNumbers = False
         self.lineSeparator = u'\n'
         self.listItemSpacer = u' '
+        self.normalizedVarNames = True
         self.omitLastSemicolon = True
         self.paranthesisSpacer = u' '
         self.propertyNameSpacer = u' '
+        self.resolveVariables = True
         self.selectorCombinatorSpacer = u' '
         self.spacer = u' '
         self.validOnly = False # should not be changed currently!!!
-        
+
     def useMinified(self):
         """Set options resulting in a minified stylesheet.
-        
+
         You may want to set preferences with this convenience method
         and set settings you want adjusted afterwards.
         """
@@ -141,7 +151,7 @@ class Preferences(object):
         self.indent = u''
         self.keepComments = False
         self.keepEmptyRules = False
-        self.keepUnkownAtRules = False
+        self.keepUnknownAtRules = False
         self.keepUsedNamespaceRulesOnly = True
         self.lineNumbers = False
         self.lineSeparator = u''
@@ -153,22 +163,28 @@ class Preferences(object):
         self.spacer = u''
         self.validOnly = False
 
+    @Deprecated(u'Use ``keepUnknownAtRules`` (note the typo) instead - ignored.')
+    def __keepUnkownAtRules(self, ignored=None):
+        pass
+
+    keepUnkownAtRules = property(__keepUnkownAtRules, __keepUnkownAtRules,
+                                  doc=u'Use keepUnknownAtRules instead!')
 
 class Out(object):
     """A simple class which makes appended items available as a combined string"""
     def __init__(self, ser):
         self.ser = ser
         self.out = []
-    
+
     def _remove_last_if_S(self):
         if self.out and not self.out[-1].strip():
             # remove trailing S
             del self.out[-1]
-    
-    def append(self, val, typ=None, space=True, keepS=False, indent=False, 
+
+    def append(self, val, typ=None, space=True, keepS=False, indent=False,
                lineSeparator=False):
         """Appends val. Adds a single S after each token except as follows:
-        
+
         - typ COMMENT
             uses cssText depending on self.ser.prefs.keepComments
         - typ "Property", cssutils.css.CSSRule.UNKNOWN_RULE
@@ -191,13 +207,12 @@ class Out(object):
             add ``*spacer`` except ``space=False``
         """
         prefspace = self.ser.prefs.spacer
-        
         if val or typ in ('STRING', 'URI'):
             # PRE
             if 'COMMENT' == typ:
                 if self.ser.prefs.keepComments:
                     val = val.cssText
-                else: 
+                else:
                     return
             elif hasattr(val, 'cssText'):
                 val = val.cssText
@@ -214,9 +229,9 @@ class Out(object):
                     del self.out[-1]
             elif 'STRING' == typ:
                 # may be empty but MUST not be None
-                if val is None: 
+                if val is None:
                     return
-                val = helper.string(val) 
+                val = helper.string(val)
                 if not prefspace:
                     self._remove_last_if_S()
             elif 'URI' == typ:
@@ -225,12 +240,15 @@ class Out(object):
                 val = self.ser._hash(val)
             elif val in u'+>~,:{;)]/':
                 self._remove_last_if_S()
-                
+
             # APPEND
             if indent:
                 self.out.append(self.ser._indentblock(val, self.ser._level+1))
             else:
+                if val.endswith(u' '):
+                    self._remove_last_if_S()
                 self.out.append(val)
+
             # POST
             if lineSeparator:
                 # Property , ...
@@ -238,6 +256,9 @@ class Out(object):
             elif val in u'+>~': # enclose selector combinator
                 self.out.insert(-1, self.ser.prefs.selectorCombinatorSpacer)
                 self.out.append(self.ser.prefs.selectorCombinatorSpacer)
+            elif u')' == val and not keepS: # CHAR funcend
+                # TODO: pref?
+                self.out.append(u' ')
             elif u',' == val: # list
                 self.out.append(self.ser.prefs.listItemSpacer)
             elif u':' == val: # prop
@@ -252,7 +273,7 @@ class Out(object):
                 if typ != 'STRING' and not self.ser.prefs.spacer and \
                    self.out and not self.out[-1].endswith(u' '):
                     self.out.append(u' ')
-        
+
     def value(self, delim=u'', end=None, keepS=False):
         "returns all items joined by delim"
         if not keepS:
@@ -260,7 +281,7 @@ class Out(object):
         if end:
             self.out.append(end)
         return delim.join(self.out)
-    
+
 
 class CSSSerializer(object):
     """Serialize a CSSStylesheet and its parts.
@@ -277,7 +298,7 @@ class CSSSerializer(object):
             prefs = Preferences()
         self.prefs = prefs
         self._level = 0 # current nesting level
-        
+
         # TODO:
         self._selectors = [] # holds SelectorList
         self._selectorlevel = 0 # current specificity nesting level
@@ -333,11 +354,28 @@ class CSSSerializer(object):
         else:
             return val
 
+    def _possiblezero(self, cssvalue, type_, val):
+        """
+        Formats to "0" is possible/allowed. 
+        """
+        if type_ in ('DIMENSION', 'NUMBER', 'PERCENTAGE'):
+            n, d = cssvalue._getNumDim(val)
+            if 0 == n:
+                if cssvalue.primitiveType in cssvalue._lengthtypes:
+                    # 0 if zero value
+                    val = u'0'
+                else:
+                    val = u'0' + d
+            else:
+                val = unicode(n) + d
+                
+        return val
+
     def _valid(self, x):
         "checks items valid property and prefs.validOnly"
-        return not self.prefs.validOnly or (self.prefs.validOnly and 
+        return not self.prefs.validOnly or (self.prefs.validOnly and
                                             x.valid)
-    
+
     def do_CSSStyleSheet(self, stylesheet):
         """serializes a complete CSSStyleSheet"""
         useduris = stylesheet._getUsedURIs()
@@ -347,19 +385,19 @@ class CSSSerializer(object):
                rule.NAMESPACE_RULE == rule.type and\
                rule.namespaceURI not in useduris and (
                     rule.prefix or None not in useduris):
-                continue 
+                continue
 
             cssText = rule.cssText
             if cssText:
                 out.append(cssText)
         text = self._linenumnbers(self.prefs.lineSeparator.join(out))
-        
+
         # get encoding of sheet, defaults to UTF-8
         try:
             encoding = stylesheet.cssRules[0].encoding
         except (IndexError, AttributeError):
             encoding = 'UTF-8'
-        
+
         return text.encode(encoding, 'escapecss')
 
     def do_CSSComment(self, rule):
@@ -384,6 +422,32 @@ class CSSSerializer(object):
         else:
             return u''
 
+    def do_CSSVariablesRule(self, rule):
+        """
+        serializes CSSVariablesRule
+
+        media
+            TODO
+        variables
+            CSSStyleDeclaration
+
+        + CSSComments
+        """
+        variablesText = rule.variables.cssText
+
+        if variablesText and rule.wellformed and not self.prefs.resolveVariables:
+            out = Out(self)
+            out.append(self._atkeyword(rule, u'@variables'))
+            for item in rule.seq:
+                # assume comments {
+                out.append(item.value, item.type)
+            out.append(u'{')
+            out.append(u'%s%s}' % (variablesText, self.prefs.lineSeparator),
+                       indent=1)
+            return out.value()
+        else:
+            return u''
+
     def do_CSSFontFaceRule(self, rule):
         """
         serializes CSSFontFaceRule
@@ -397,14 +461,14 @@ class CSSSerializer(object):
 
         if styleText and rule.wellformed:
             out = Out(self)
-            out.append(self._atkeyword(rule, u'@font-face'))   
+            out.append(self._atkeyword(rule, u'@font-face'))
             for item in rule.seq:
                 # assume comments {
-                out.append(item.value, item.type)            
+                out.append(item.value, item.type)
             out.append(u'{')
             out.append(u'%s%s}' % (styleText, self.prefs.lineSeparator),
-                       indent=1)            
-            return out.value()            
+                       indent=1)
+            return out.value()
         else:
             return u''
 
@@ -417,14 +481,14 @@ class CSSSerializer(object):
         media
             optional cssutils.stylesheets.medialist.MediaList
         name
-            optional string 
+            optional string
 
         + CSSComments
         """
         if rule.wellformed:
             out = Out(self)
             out.append(self._atkeyword(rule, u'@import'))
-            
+
             for item in rule.seq:
                 typ, val = item.type, item.value
                 if 'href' == typ:
@@ -439,7 +503,7 @@ class CSSSerializer(object):
                     # media
                     mediaText = self.do_stylesheets_medialist(val)
                     if mediaText and mediaText != u'all':
-                        out.append(mediaText)                
+                        out.append(mediaText)
                 elif 'name' == typ:
                     out.append(val, 'STRING')
                 else:
@@ -462,18 +526,18 @@ class CSSSerializer(object):
         """
         if rule.wellformed:
             out = Out(self)
-            out.append(self._atkeyword(rule, u'@namespace'))            
+            out.append(self._atkeyword(rule, u'@namespace'))
             for item in rule.seq:
                 typ, val = item.type, item.value
                 if 'namespaceURI' == typ:
                     out.append(val, 'STRING')
                 else:
                     out.append(val, typ)
-                    
+
             return out.value(end=u';')
         else:
             return u''
-        
+
     def do_CSSMediaRule(self, rule):
         """
         serializes CSSMediaRule
@@ -487,15 +551,15 @@ class CSSSerializer(object):
             return u''
 
         # @media
-        out = [self._atkeyword(rule, u'@media')] 
+        out = [self._atkeyword(rule, u'@media')]
         if not len(self.prefs.spacer):
             # for now always with space as only webkit supports @mediaall?
             out.append(u' ')
-        else: 
+        else:
             out.append(self.prefs.spacer) # might be empty
-        
+
         out.append(self.do_stylesheets_medialist(rule.media))
-        
+
         # name, seq contains content after name only (Comments)
         if rule.name:
             out.append(self.prefs.spacer)
@@ -504,12 +568,12 @@ class CSSSerializer(object):
             for item in rule.seq:
                 nameout.append(item.value, item.type)
             out.append(nameout.value())
-            
+
         #  {
         out.append(self.prefs.paranthesisSpacer)
-        out.append(u'{')                      
+        out.append(u'{')
         out.append(self.prefs.lineSeparator)
-        
+
         # rules
         rulesout = []
         for r in rule.cssRules:
@@ -519,12 +583,12 @@ class CSSSerializer(object):
                 rulesout.append(self._indentblock(rtext, self._level + 1))
                 rulesout.append(self.prefs.lineSeparator)
         if not self.prefs.keepEmptyRules and not u''.join(rulesout).strip():
-            return u''           
+            return u''
         out.extend(rulesout)
-        
+
         #     }
         out.append(u'%s}' % ((self._level + 1) * self.prefs.indent))
-        
+
         return u''.join(out)
 
     def do_CSSPageRule(self, rule):
@@ -542,11 +606,11 @@ class CSSSerializer(object):
         if styleText and rule.wellformed:
             out = Out(self)
             out.append(self._atkeyword(rule, u'@page'))
-            out.append(rule.selectorText)                            
-            out.append(u'{')            
+            out.append(rule.selectorText)
+            out.append(u'{')
             out.append(u'%s%s}' % (styleText, self.prefs.lineSeparator),
                        indent=1)
-            return out.value()            
+            return out.value()
         else:
             return u''
 
@@ -559,17 +623,17 @@ class CSSSerializer(object):
             else:
                 out.append(item.value, item.type)
         return out.value()
-        
+
     def do_CSSUnknownRule(self, rule):
         """
         serializes CSSUnknownRule
         anything until ";" or "{...}"
         + CSSComments
         """
-        if rule.wellformed and self.prefs.keepUnkownAtRules:
+        if rule.wellformed and self.prefs.keepUnknownAtRules:
             out = Out(self)
-            out.append(rule.atkeyword)  
-                         
+            out.append(rule.atkeyword)
+
             stacks = []
             for item in rule.seq:
                 typ, val = item.type, item.value
@@ -579,7 +643,7 @@ class CSSSerializer(object):
                     stackblock = stacks.pop().value()
                     if stackblock:
                         val = self._indentblock(
-                               stackblock + self.prefs.lineSeparator + val, 
+                               stackblock + self.prefs.lineSeparator + val,
                                min(1, len(stacks)+1))
                     else:
                         val = self._indentblock(val, min(1, len(stacks)+1))
@@ -588,12 +652,12 @@ class CSSSerializer(object):
                     stacks[-1].append(val, typ)
                 else:
                     out.append(val, typ)
-                    
+
                 # POST
                 if u'{' == val:
                     # new stack level
                     stacks.append(Out(self))
-            
+
             return out.value()
         else:
             return u''
@@ -626,12 +690,12 @@ class CSSSerializer(object):
                 elif self._selectorlevel > 0:
                     self._selectorlevel -= 1
             else:
-                # save new reference                
+                # save new reference
                 self._selectors.append(rule.selectorList)
                 self._selectorlevel = 0
-        
+
         # TODO ^ RESOLVE!!!!
-        
+
         selectorText = self.do_css_SelectorList(rule.selectorList)
         if not selectorText or not rule.wellformed:
             return u''
@@ -660,7 +724,7 @@ class CSSSerializer(object):
         "comma-separated list of Selectors"
         # does not need Out() as it is too simple
         if selectorlist.wellformed:
-            out = [] 
+            out = []
             for part in selectorlist.seq:
                 if isinstance(part, cssutils.css.Selector):
                     out.append(part.selectorText)
@@ -670,13 +734,13 @@ class CSSSerializer(object):
             return sep.join(out)
         else:
             return u''
-                  
+
     def do_css_Selector(self, selector):
         """
         a single Selector including comments
-        
+
         an element has syntax (namespaceURI, name) where namespaceURI may be:
-        
+
         - cssutils._ANYNS => ``*|name``
         - None => ``name``
         - u'' => ``|name``
@@ -684,14 +748,14 @@ class CSSSerializer(object):
         """
         if selector.wellformed:
             out = Out(self)
-            
+
             DEFAULTURI = selector._namespaces.get('', None)
             for item in selector.seq:
                 typ, val = item.type, item.value
                 if type(val) == tuple:
                     # namespaceURI|name (element or attribute)
                     namespaceURI, name = val
-                    if DEFAULTURI == namespaceURI or (not DEFAULTURI and 
+                    if DEFAULTURI == namespaceURI or (not DEFAULTURI and
                                                       namespaceURI is None):
                         out.append(name, typ, space=False)
                     else:
@@ -703,22 +767,53 @@ class CSSSerializer(object):
                                                     namespaceURI)
                             except IndexError:
                                 prefix = u''
-                        
+
                         out.append(u'%s|%s' % (prefix, name), typ, space=False)
                 else:
                     out.append(val, typ, space=False, keepS=True)
-            
+
             return out.value()
-        else: 
+        else:
+            return u''
+
+    def do_css_CSSVariablesDeclaration(self, variables):
+        """Variables of CSSVariableRule."""
+        if len(variables.seq) > 0:
+            out = Out(self)
+
+            lastitem = len(variables.seq) - 1
+            for i, item in enumerate(variables.seq):
+                type_, val = item.type, item.value
+                if u'var' == type_:
+                    name, cssvalue = val
+                    if self.prefs.normalizedVarNames:
+                        name = normalize(name)
+                    out.append(name)
+                    out.append(u':')
+                    out.append(cssvalue.cssText)
+                    if i < lastitem or not self.prefs.omitLastSemicolon:
+                        out.append(u';')
+
+                elif isinstance(val, cssutils.css.CSSComment):
+                    # CSSComment
+                    out.append(val, 'COMMENT')
+                    out.append(self.prefs.lineSeparator)
+                else:
+                    out.append(val.cssText, type_)
+                    out.append(self.prefs.lineSeparator)
+
+            return out.value().strip()
+
+        else:
             return u''
 
     def do_css_CSSStyleDeclaration(self, style, separator=None):
         """
         Style declaration of CSSStyleRule
         """
-#        # TODO: use Out()
-        
-        # may be comments only       
+        # TODO: use Out()
+
+        # may be comments only
         if len(style.seq) > 0:
             if separator is None:
                 separator = self.prefs.lineSeparator
@@ -729,8 +824,8 @@ class CSSSerializer(object):
             else:
                 # only effective ones
                 _effective = style.getProperties()
-                seq = [item for item in style.seq 
-                         if (isinstance(item.value, cssutils.css.Property) 
+                seq = [item for item in style.seq
+                         if (isinstance(item.value, cssutils.css.Property)
                              and item.value in _effective)
                          or not isinstance(item.value, cssutils.css.Property)]
 
@@ -770,11 +865,11 @@ class CSSSerializer(object):
         """
         Style declaration of CSSStyleRule
 
-        Property has a seqs attribute which contains seq lists for             
+        Property has a seqs attribute which contains seq lists for
         name, a CSSvalue and a seq list for priority
         """
         # TODO: use Out()
-        
+
         out = []
         if property.seqs[0] and property.wellformed and self._valid(property):
             nameseq, cssvalue, priorityseq = property.seqs
@@ -816,7 +911,7 @@ class CSSSerializer(object):
         """
         a Properties priority "!" S* "important"
         """
-        # TODO: use Out()       
+        # TODO: use Out()
         out = []
         for part in priorityseq:
             if hasattr(part, 'cssText'): # comments
@@ -844,8 +939,8 @@ class CSSSerializer(object):
                     # S must be kept! in between values but no extra space
                     out.append(val, type_)
 
-            return out.value() 
-                        
+            return out.value()
+
     def do_css_CSSPrimitiveValue(self, cssvalue):
         """Serialize a CSSPrimitiveValue"""
         if not cssvalue:
@@ -854,44 +949,64 @@ class CSSSerializer(object):
             out = Out(self)
             for item in cssvalue.seq:
                 type_, val = item.type, item.value
-                if type_ in ('DIMENSION', 'NUMBER', 'PERCENTAGE'):
-                    n, d = cssvalue._getNumDim(val)
-                    if 0 == n:
-                        if cssvalue.primitiveType in cssvalue._lengthtypes:
-                            # 0 if zero value
-                            val = u'0'
-                        else:
-                            val = u'0' + d
-                    else:
-                        val = unicode(n) + d
+                val = self._possiblezero(cssvalue, type_, val)
                 out.append(val, type_)
-                
-            return out.value() 
-            
-    def do_css_RGBColor(self, cssvalue):
-        """Serialize a RGBColor value"""
+
+            return out.value()
+
+    def do_css_CalcValue(self, cssvalue):
+        """Serialize a CalcValue."""
+        return self.do_css_CSSPrimitiveValue(cssvalue)
+
+    def do_css_FunctionValue(self, cssvalue):
+        """Serialize a CSS function value"""
         if not cssvalue:
             return u''
         else:
             out = Out(self)
-            unary = None
             for item in cssvalue.seq:
                 type_, val = item.type, item.value
-                
-#                # prepare
-#                if 'CHAR' == type_ and val in u'+-':
-#                    # save - for next round                
-#                    if u'-' == val:
-#                        # omit +
-#                        unary = val
-#                    continue
-#                elif unary:
-#                    val = unary + val.cssText
-#                    unary = None
-                    
                 out.append(val, type_)
-            
-            return out.value() 
+
+            return out.value()
+
+    def do_css_RGBColor(self, cssvalue):
+        """Serialize a RGBColor value"""
+        return self.do_css_FunctionValue(cssvalue)
+
+    def do_css_ExpressionValue(self, cssvalue):
+        """Serialize an ExpressionValue (IE only),
+        should at least keep the original syntax"""
+        if not cssvalue:
+            return u''
+        else:
+            out = Out(self)
+            for item in cssvalue.seq:
+                type_, val = item.type, item.value
+                val = self._possiblezero(cssvalue, type_, val)
+                # do no send type_ so no special cases!
+                out.append(val, None, space=False)
+
+            return out.value()
+
+    def do_css_CSSVariable(self, variable):
+        """Serializes a CSSVariable"""
+        if not variable:
+            return u''
+        else:
+            out = Out(self)
+            v = variable.value
+            if self.prefs.resolveVariables and v:
+                # resolve variable
+                out.append(v)
+
+            else:
+                # keep var(NAME)
+                for item in variable.seq:
+                    type_, val = item.type, item.value
+                    out.append(val, type_)
+
+            return out.value()
 
     def do_stylesheets_medialist(self, medialist):
         """
@@ -923,29 +1038,3 @@ class CSSSerializer(object):
             return u' '.join(out)
         else:
             return u''
-
-
-    def do_css_ExpressionValue(self, cssvalue):
-        """Serialize an ExpressionValue (IE only), 
-        should at least keep the original syntax"""
-        if not cssvalue:
-            return u''
-        else:
-            out = Out(self)
-            for item in cssvalue.seq:
-                type_, val = item.type, item.value
-                if type_ in ('DIMENSION', 'NUMBER', 'PERCENTAGE'):
-                    n, d = cssvalue._getNumDim(val)
-                    if 0 == n:
-                        if cssvalue.primitiveType in cssvalue._lengthtypes:
-                            # 0 if zero value
-                            val = u'0'
-                        else:
-                            val = u'0' + d
-                    else:
-                        val = unicode(n) + d
-                # do no send type_ so no special cases!
-                out.append(val, None, space=False)
-                
-            return out.value() 
-            
\ No newline at end of file
